<?php
// This file is part of Moodle - http://moodle.org/
//
// Moodle is free software: you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
//
// Moodle is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with Moodle.  If not, see <http://www.gnu.org/licenses/>.

/**
 * Access restriction setting script in "My Media".
 *
 * @package   local_yumymedia
 * @copyright (C) 2016-2025 Yamaguchi University <gh-cc@mlex.cc.yamaguchi-u.ac.jp>
 * @license   http://www.gnu.org/copyleft/gpl.html GNU GPL v3 or later
 */

require_once(dirname(dirname(dirname(__FILE__))) . '/config.php');
require_once(dirname(dirname(dirname(__FILE__))) . '/local/yukaltura/locallib.php');
require_once(dirname(dirname(dirname(__FILE__))) . '/local/yukaltura/API/KalturaClient.php');

header('Access-Control-Allow-Origin: *');
header('Cache-Control: no-cache');

global $USER, $SESSION, $DB;

defined('MOODLE_INTERNAL') || die();

$entryid = required_param('entryid', PARAM_TEXT);
$page = required_param('page', PARAM_INT);
$sort = optional_param('sort', 'recent', PARAM_TEXT);

$mymedia = get_string('heading_mymedia', 'local_yumymedia');
$PAGE->set_context(context_system::instance());
$header = format_string($SITE->shortname).": $mymedia";

$PAGE->set_url('/local/yumymedia/access_media.php');
$PAGE->set_course($SITE);

require_login();

$PAGE->set_pagetype('yumymedia-index');
$PAGE->set_pagelayout('frontpage');
$PAGE->set_title($header);
$PAGE->set_heading($header);
$PAGE->add_body_class('mymedia-index');
$PAGE->requires->css('/local/yumymedia/css/yumymedia.css');
$PAGE->requires->js_call_amd('local_yumymedia/accessrestriction', 'init', array());

$kaltura = new yukaltura_connection();
$connection = $kaltura->get_connection(false, true, KALTURA_SESSION_LENGTH);
$context = context_user::instance($USER->id);

if (!$connection) {
    $url = new moodle_url('/admin/settings.php', array('section' => 'local_yukaltura'));
    throw new moodle_exeption('conn_failed', 'local_yukaltura', $url);
}

// Get publiser name, admin secret, hostname, parter id.
$publishername = local_yukaltura_get_publisher_name();
$secret = local_yukaltura_get_admin_secret();
$kalturahost = local_yukaltura_get_host();
$partnerid = local_yukaltura_get_partner_id();
$uiconfid = local_yukaltura_get_player_uiconf('player_mymedia');
$expiry = KALTURA_SESSION_LENGTH;

$playerstudio = "html5";
$playertype = local_yukaltura_get_player_type($uiconfid, $connection);
if ($playertype == KALTURA_TV_PLATFORM_STUDIO) {
       $playerstudio = "ovp";
}

echo $OUTPUT->header();

$media = $connection->media->get($entryid);

if ($media == null || $media->status == KalturaEntryStatus::DELETED) {
    echo get_string('access_media_not_exist', 'local_yumymedia');
} else if ($media->userId != $USER->username) {
    echo get_string('access_media_failed_not_owner', 'local_yumymedia');
} else {
    // Star kaltura session.
    $ks = $connection->session->start($secret, $publishername, KalturaSessionType::ADMIN, $partnerid, $expiry);

    if ($ks == null || $ks == '') {
        $url = new moodle_url('/local/yumymedia/access_media.php', array('entryid' => $entryid));
         throw new moodle_exception('kaltura_session_failed', 'local_yukaltura', $url);
    } else {
        echo html_writer::start_tag('h3');
        echo get_string('access_media_title', 'local_yumymedia');
        echo html_writer::end_tag('h3');

        $internalcontrolprofile = local_yukaltura_get_internal_access_control($connection);
        $defaultcontrolprofile = local_yukaltura_get_default_access_control($connection);
        $currentcontrolid = $media->accessControlId;
        $url = new moodle_url('/local/yumymedia/yumymedia.php', array('page' => $page, 'sort' => $sort));

        $renderer = $PAGE->get_renderer('local_yumymedia');

        echo $renderer->create_media_details_table_markup($media);
        echo $renderer->create_hidden_input_markup($kalturahost, $ks, $entryid, $partnerid, $uiconfid,
                                                   $playerstudio, $url, $currentcontrolid);
        echo $renderer->create_embed_code_markup();

        if ($media != null && !is_null($internalcontrolprofile) && !is_null($defaultcontrolprofile)) {
                echo $renderer->create_access_control_markup($defaultcontrolprofile, $internalcontrolprofile,
                                                             $currentcontrolid);
        }

        echo $renderer->create_access_back_markup($url);
    }
}

echo $OUTPUT->footer();

